@@ -6,6 +6,9 @@ require URI::_query;
 use strict;
 use URI::Escape qw(uri_unescape);
 
+my $ACHAR = $URI::uric;  $ACHAR =~ s,\\[/?],,g;
+my $PCHAR = $URI::uric;  $PCHAR =~ s,\\[?],,g;
+
 sub authority
 {
     my $self = shift;
@@ -16,7 +19,7 @@ sub authority
 	$$self = $1;
 	my $rest = $3;
 	if (defined $auth) {
-	    $auth =~ s/([^$URI::achar])/$URI::Escape::escapes{$1}/go;
+	    $auth =~ s/([^$ACHAR])/$URI::Escape::escapes{$1}/go;
 	    $$self .= "//$auth";
 	} elsif ($rest =~ m,^//,) {
 	    warn "Path starting with double slash is confusing";
@@ -37,7 +40,7 @@ sub path
 	my $rest = $3;
 	my $new_path = shift;
 	$new_path = "" unless defined $new_path;
-	$new_path =~ s/([^$URI::ppchar])/$URI::Escape::escapes{$1}/go;
+	$new_path =~ s/([^$PCHAR])/$URI::Escape::escapes{$1}/go;
 	if (length($$self)) {
 	    $$self .= "/" if length($new_path) && $new_path !~ m,^/,;	    
 	} else {
@@ -54,6 +57,7 @@ sub path
     $2;
 }
 
+
 sub abs_path
 {
     my $self = shift;
@@ -71,6 +75,7 @@ sub abs_path_query
     $tmp;
 }
 
+
 sub path_segments
 {
     my $self = shift;
@@ -80,16 +85,16 @@ sub path_segments
 	for (@arg) {
 	    if (ref($_)) {
 		my @seg = @$_;
-		$seg[0] =~ s/([^$URI::pchar])/$URI::Escape::escapes{$1}/go;
+		for (@seg) { s/;/%3B/g; }
 		$_ = join(";", @seg);
 	    } else {
-		s/([^$URI::pchar])/$URI::Escape::escapes{$1}/go;
+		s/;/%3B/g;
 	    }
+	    s,/,%2F,g;
 	}
 	$self->path(join("/", @arg));
     }
     return $path unless wantarray;
-    $path = "/$path" if defined($self->authority) && $path !~ m,^/,;
     map {/;/ ? _split_segment($_) : uri_unescape($_) } split('/', $path, -1);
 }
 
@@ -105,33 +110,27 @@ sub _split_segment
 sub abs
 {
     my $self = shift;
-    my $abs = $self->clone;
-    my $base = shift || return $abs;
+    my $base = shift || die "Missing base argument";
     my $allow_scheme = shift;
 
-    $base = URI->new($base) unless ref $base;
-
-    #my($scheme, $authority, $path, $query, $fragment) =
-    #   @{$self}{qw(scheme authority path query fragment)};
-    my $scheme    = $self->scheme;
-    my $authority = $self->authority;
-    my $path      = $self->path;
-    my $query     = $self->query;
-    my $fragment  = $self->fragment;
-
-    if ($scheme) {
-	return $abs unless $allow_scheme;
-	return $abs if lc($scheme) ne lc($base->scheme);
+    if (my $scheme = $self->scheme) {
+	return $self unless $allow_scheme;
+	$base = URI->new($base) unless ref $base;
+	return $self unless lc($scheme) eq lc($base->scheme);
     }
+
+    $base = URI->new($base) unless ref $base;
+    my $abs = $self->clone;
     $abs->scheme($base->scheme);
-    return $abs if defined $authority;
+    return $abs if $$self =~ m,^(?:$URI::scheme_re:)?//,;
     $abs->authority($base->authority);
+
+    my $path = $self->path;
     return $abs if $path =~ m,^/,;
 
-    if (!length($path) && !defined($query)) {
-	# we are empty, reference to base (all modifications to $abs wasted)
-	$abs = $base->clone;
-	$abs->fragment($fragment);
+    if (!length($path) && $$self !~ /\?/) {
+	my $abs = $base->clone;
+	$abs->fragment($self->fragment);
 	return $abs;
     }
 
@@ -164,9 +163,8 @@ sub abs
 # The oposite of $url->abs.  Return a URI which is much relative as possible
 sub rel {
     my $self = shift;
-    my $base = shift;
+    my $base = shift || die "Missing base argument";
     my $rel = $self->clone;
-    return $rel unless $base;
     $base = URI->new($base) unless ref $base;
 
     #my($scheme, $auth, $path) = @{$rel}{qw(scheme authority path)};
